unit UdpTcpIpc_TestApp_Main_Delphi;

interface

uses
{$IFDEF LINUX}
  QComCtrls, QStdCtrls, QControls, QExtCtrls, QForms,
{$ELSE}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Spin, { MemCheck, }
{$ENDIF}
  SysUtils, Classes, HxTcpIpcUdp;

type
  TfrmMain = class(TForm)
    mmo: TMemo;
    Panel1: TPanel;
    btnTcpServer: TButton;
    btnTcpClient: TButton;
    edtTcpText: TEdit;
    btnTcpSend: TButton;
    Label1: TLabel;
    Panel2: TPanel;
    Label2: TLabel;
    btnUdp: TButton;
    edtUdpText: TEdit;
    btnUdpSend: TButton;
    Panel3: TPanel;
    Label3: TLabel;
    btnIpcServer: TButton;
    btnIpcClient: TButton;
    edtIpcText: TEdit;
    btnIpcSend: TButton;
    speUdpListeningPort: TSpinEdit;
    speUdpDestPort: TSpinEdit;
    Label4: TLabel;
    Label5: TLabel;
    btnUdpReply: TButton;
    speTcpListeningPort: TSpinEdit;
    speTcpDestPort: TSpinEdit;
    speIpcListeningPort: TSpinEdit;
    speIpcDestPort: TSpinEdit;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    speIpcClient: TSpinEdit;
    Label10: TLabel;
    Label11: TLabel;
    speTcpClient: TSpinEdit;
    procedure btnIpcServerClick(Sender: TObject);
    procedure btnIpcClientClick(Sender: TObject);
    procedure btnIpcSendClick(Sender: TObject);
    procedure btnTcpServerClick(Sender: TObject);
    procedure btnTcpClientClick(Sender: TObject);
    procedure btnTcpSendClick(Sender: TObject);
    procedure btnUdpClick(Sender: TObject);
    procedure btnUdpSendClick(Sender: TObject);
    procedure btnUdpReplyClick(Sender: TObject);
    procedure speUdpDestPortChange(Sender: TObject);
    procedure speUdpListeningPortChange(Sender: TObject);
    procedure speTcpDestPortChange(Sender: TObject);
    procedure speTcpListeningPortChange(Sender: TObject);
    procedure speIpcListeningPortChange(Sender: TObject);
    procedure speIpcDestPortChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    svrIpc: THxIpcServer;
    clntIpc: THxIpcClient;
    svrTCP: THxTcpServer;
    clntTCP: THxTcpClient;
    udp: THxUdp;
    procedure HandlesvrIpcDataAvailable(ptr: pointer; iSize, iDescr: integer);
    procedure HandlesvrIpcClientAborted(iDescr: integer);
    procedure HandlesvrIpcNewClient(iDescr: integer);
    procedure HandleNotify(S: string);
    procedure HandleClientConnectionClosed;
    procedure HandleClientDataAvailable(ptr: pointer; iSize: integer);
    procedure HandleUdpData(ptr: pchar; iSize: integer; sRemoteIP,
      sPort: string);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$IFDEF LINUX}
{$R *.xfm}
{$ELSE}
{$R *.DFM}
{$ENDIF}


procedure TfrmMain.HandlesvrIpcDataAvailable( ptr: pointer; iSize, iDescr: integer );
var
  s: string;
begin
  s := StrPas( pchar( ptr ));
  mmo.Lines.Add(Format('Data from client: %d, message: %s, size: %d',
                      [iDescr,s, iSize]));
end;

procedure TfrmMain.HandlesvrIpcClientAborted( iDescr: integer );
begin
  mmo.Lines.Add(Format('Client: %d aborted',[iDescr]));
end;

procedure TfrmMain.HandlesvrIpcNewClient( iDescr: integer );
var
  sPort: string;
begin
  if Assigned( svrIpc )then
    sPort := svrIpc.ClientPort[ iDescr ]
  else if Assigned( svrTcp )then
    sPort := svrTcp.ClientPort[ iDescr ]
  else
    sPort := '??????';
  mmo.Lines.Add(Format('New Client: %d, Client-Port: %s',[iDescr, sPort]));

end;

procedure TfrmMain.HandleNotify( S: string );
begin
  mmo.Lines.Add(Format('Notify message: %s', [s]));
end;

procedure TfrmMain.btnIpcServerClick(Sender: TObject);
begin
  if Assigned( svrIpc )then
    exit;
  svrIpc := THxIpcServer.Create;
  svrIpc.Port := speIpcListeningPort.Text;
  svrIpc.OnDataAvailable := HandlesvrIpcDataAvailable;
  svrIpc.OnClientAborted := HandlesvrIpcClientAborted;
  svrIpc.OnNewClient := HandlesvrIpcNewClient;
  svrIpc.OnNotify := HandleNotify;
  svrIpc.Listen;

  btnIpcSend.Enabled := True;
//  btnIpcClient.Enabled := False;
end;

procedure TfrmMain.HandleClientDataAvailable( ptr: pointer; iSize: integer );
var
  s: string;
begin
  s := StrPas( pchar( ptr ));
  mmo.Lines.Add(Format('Data from server, message: %s, size: %d', [s, iSize]));
end;

procedure TfrmMain.HandleClientConnectionClosed;
begin
  mmo.Lines.Add('Sever logged off');
end;

procedure TfrmMain.btnIpcClientClick(Sender: TObject);
begin
  if Assigned( clntIpc )then
  begin
    if not( clntIpc.Connected )then
    begin
      clntIpc.Port := speIpcDestPort.Text;
      if clntIpc.Connect then
      begin
        mmo.Lines.Add('connected');
        btnIpcSend.Enabled := True;
      end
      else
        mmo.Lines.Add('Not connected');
    end;
    exit;
  end;

  clntIpc := THxIpcClient.Create;
  clntIpc.ServerAddress := 'localhost';
  clntIpc.Port := speIpcDestPort.Text;
  clntIpc.OnNotify := HandleNotify;
  clntIpc.OnDataAvailable := HandleClientDataAvailable;
  clntIpc.OnConnectionClosed := HandleClientConnectionClosed;
  if clntIpc.Connect then
  begin
    mmo.Lines.Add('connected');
    btnIpcSend.Enabled := True;
  end
  else
    mmo.Lines.Add('Not connected');
    
  speIpcClient.Enabled := False;
  btnIpcServer.Enabled := False;
end;

procedure TfrmMain.btnIpcSendClick(Sender: TObject);
begin
  if Assigned(clntIpc)then
    clntIpc.SendString( edtIpcText.Text )
  else if Assigned( svrIpc )then
    svrIpc.SendString( edtIpcText.Text, speIpcClient.Value );
end;

procedure TfrmMain.btnTcpServerClick(Sender: TObject);
begin
  if Assigned( svrTCP )then
    exit;

  svrTCP := THxTcpServer.Create;
  svrTCP.Port := speTcpListeningPort.Text;
  svrTCP.OnDataAvailable := HandlesvrIpcDataAvailable;
  svrTCP.OnClientAborted := HandlesvrIpcClientAborted;
  svrTCP.OnNewClient := HandlesvrIpcNewClient;
  svrTCP.OnNotify := HandleNotify;
  svrTCP.Listen;

//  btnTcpClient.Enabled := False;
  speTcpDestPort.Enabled := False;
  btnTcpSend.Enabled := True;
end;

procedure TfrmMain.btnTcpClientClick(Sender: TObject);
begin
  if Assigned( clntTcp )then
  begin
    if not( clntTcp.Connected )then
    begin
      clntTcp.Port := speTcpDestPort.Text;
      clntTcp.ServerAddress := 'localhost';
      if clntTcp.Connect then
      begin
        mmo.Lines.Add('TCP connected');
        btnTcpSend.Enabled := True;
      end
      else
        mmo.Lines.Add('TCP Not connected');
    end;
    exit;
  end;

  clntTcp := THxTcpClient.Create;
  clntTcp.ServerAddress := 'localhost';
  clntTcp.Port := speTcpDestPort.Text;
  clntTcp.OnNotify := HandleNotify;
  clntTcp.OnDataAvailable := HandleClientDataAvailable;
  clntTcp.OnConnectionClosed := HandleClientConnectionClosed;
  if clntTcp.Connect then
  begin
    mmo.Lines.Add('Tcp connected');
    btnTcpSend.Enabled := True;
  end
  else
    mmo.Lines.Add('Tcp Not connected');

  btnTcpServer.Enabled := False;
  speTcpListeningPort.Enabled := False;
  speTcpClient.Enabled := False;
end;

procedure TfrmMain.btnTcpSendClick(Sender: TObject);
begin
  if Assigned( clntTcp )then
    clntTcp.SendString( edtTcpText.Text )
  else if Assigned( svrTcp )then
    svrTcp.SendString( edtTcpText.Text, speTcpClient.Value );
end;

procedure TfrmMain.HandleUdpData( ptr: pchar; iSize: integer;
                                sRemoteIP, sPort: string );
var
  s: string;
begin
  s := StrPas( pchar( ptr ));
  mmo.Lines.Add(Format('Data by UDP, message: %s, size: %d, '+
                       'remoteIp: %s, Port: %s', [s, iSize, sRemoteIP, sPort]));
end;

procedure TfrmMain.btnUdpClick(Sender: TObject);
begin
  if Assigned( udp )then
    exit;

//  MemChk;

  udp := THxUdp.Create;
  udp.OnNotify := HandleNotify;
  udp.OnDataAvailable := HandleUdpData;
  udp.DestinationAddress := 'localhost';
  udp.DestinationPort := speUdpDestPort.Text;
  udp.ListeningPort := speUdpListeningPort.Text;

  btnUdpSend.Enabled := True;
  btnUdpReply.Enabled := True;
end;

procedure TfrmMain.btnUdpSendClick(Sender: TObject);
begin
  if not( Assigned( udp ))then
    exit;

  udp.SendString( edtUdpText.Text );
end;

procedure TfrmMain.btnUdpReplyClick(Sender: TObject);
begin
  if not( Assigned( udp ))then
    exit;

  udp.ReplyStr( edtUdpText.Text );
end;

procedure TfrmMain.speUdpDestPortChange(Sender: TObject);
begin
  if( not( Assigned( udp )))or( speUdpDestPort.Text = '' )then
    exit;

  udp.DestinationPort := speUdpDestPort.Text;
end;

procedure TfrmMain.speUdpListeningPortChange(Sender: TObject);
begin
  if( not( Assigned( udp )))or( speUdpListeningPort.Text = '' )then
    exit;

  udp.ListeningPort := speUdpListeningPort.Text;
end;


procedure TfrmMain.speTcpDestPortChange(Sender: TObject);
begin
  if( not( Assigned( clntTcp )))or( speTcpDestPort.Text = '' )then
    exit;

  clntTCP.Port := IntToStr( speTcpDestPort.Value );
end;

procedure TfrmMain.speTcpListeningPortChange(Sender: TObject);
begin
  if( not( Assigned( svrTcp )))or( speTcpListeningPort.Text = '' )then
    exit;

  svrTCP.Port := IntToStr( speTcpListeningPort.Value );
end;

procedure TfrmMain.speIpcListeningPortChange(Sender: TObject);
begin
  if( not( Assigned( svrIpc )))or( speIpcListeningPort.Text = '' )then
    exit;

  svrIpc.Port := IntToStr( speIpcListeningPort.Value );
end;

procedure TfrmMain.speIpcDestPortChange(Sender: TObject);
begin
  if( not( Assigned( clntIpc )))or( speIpcDestPort.Text = '' )then
    exit;

  clntIpc.Port := IntToStr( speIpcDestPort.Value );
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if Assigned( udp )then
    udp.Free;

  if Assigned( clntTCP )then
    clntTCP.Free;

  if Assigned( svrTcp )then
    svrTcp.Free;

  if Assigned( svrIpc )then
    svrIpc.Free;

  if Assigned( clntIpc )then
    clntIpc.Free;
end;

end.
