{
=============================================================================
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  |
| AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    |
| IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   |
| ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR  |
| ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       |
| DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR   |
| SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER   |
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT           |
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    |
| OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  |
| DAMAGE.                                                                      |
=============================================================================
   Author:  D.J.C. Blijker.
   Date:    2-2-2003
   version: 0.9               ** This still is a BETA release!!! **
   e-mail:  c.blijker@wxs.nl
=============================================================================
This unit will work in Delphi as well in Kylix!!!

This unit contains 5 useful classes (I hope) I.e.

  THxTpcClient
  THxTcpServer
  THxIpcClient
  THxIpcServer
  THxUdp

The xxTCPxx classes are synchronous wrappers around the libc/winsock
TCP API-calls.

******************************************************************************
This means that all eventhandlers are automatically called within the context
of the main-thread.
******************************************************************************

The xxIPCxx classes are 'message based' implementations of their TCP
counterparts.
  TCP is a 'streaming protocol'. This means that message boundaries are not
  automatically preserved. The IPC protocol ensures that every single message
  that has been sent will 'trigger' one corresponding 'OnDataAvailable' event
  on 'the other side'.
IpcClients can only communicate with IpcServers..

The THxUdp class is a synchronous wrapper around the UDP API-calls
-------------------------------------------------------------------------------
THxxxClient Usage:
  - Instantiate an object.
  - Provide the port number (by name or by number)
  - Provide the server address (by name or dotted-number)
  - Provide an 'OnDataAvailable' eventhandler. (if applicable)
  - Provide a 'Notify' eventhandler. This eventhandler will be called when an
    unexpected situation is encountered.
  - Provide an 'OnConnectionClosed' eventhandler if you want a notification
    when the server aborts the connection.
  - Call the function Connect.
    the result value indicates whether the connect was succesful.
  - Call Send(string)/(Data) if you want to send a message..

THxxxServer Usage:
  - Instantiate an object.
  - Provide the port number (by name or by number)
  - Provide event handlers for 'OnDataAvailable', 'OnClientAborted' and
    'OnNewClient'.
    All these event handlers work with 'descriptors' (handles). Just ordinary
    integer numbers, which represents the connection with a client.
    'OnNewClient' will supply a descriptor for every new client connected to
    the server.
  - Call the Listen procedure.

THxUdp Usage:
  - Instantiate an object.
  - Provide valid values for the DestinationAddress and DestinationPort.
  - provide a ListeningPort. This will automatically force the udp object to
    listen on the supplied port.
  - Before calling SendString/SendData a valid destination- address/port
    combination has to be supplied.
  - Call Reply() if you want to send a message to the client from which the
    last message was received.
==============================================================================
TO DO:
  Test!!! all this stuff..

Please send comments/updates etc. to: c.blijker@wxs.nl
(If you speak dutch, please send comments in dutch..)
==============================================================================
History:

  version: 0.9  Initial release.
==============================================================================
}

unit HxTcpIpcUdp;

interface

uses
  SysUtils, Classes,
{$IFDEF LINUX}
  Libc, kernelioctl,
{$ELSE}
  Windows, WinSock,
{$ENDIF}
 SyncObjs;

const
  IPC_CLIENT = 'THxIPC_Client';
  IPC_SERVER = 'THxIPC_Server';
  IPC_SVR_VERSION = 'v 1.0';
  IPC_CL_VERSION = 'v 1.0';
  IPC_INTRO_TIMEOUT = 5; // 5 seconds..

type
  dword = longword;

  TOnNotify = procedure( sMess: string )of object;

  TBaseThread = class( TThread )
  protected
    csLock: TCriticalSection;
    sMess: string;
    FOnNotify: TOnNotify;
    procedure NotifySync;
    procedure Notify( s: string ); virtual;
    procedure ExecProc; virtual; abstract;
    procedure Execute; override;
    constructor Create;
  public
    destructor  Destroy; override;
// OnNotify will always be called in the context of the main thread!!
    property OnNotify: TOnNotify read FOnNotify write FOnNotify;
  end;

  THxBlockSocket = class;

  TOnDisconnect = procedure( blkSocket: THxBlockSocket )of object;

  THxSocketInterface = interface
    procedure Disconnect;
    function  Connect: integer;
    function  SendData( ptr: pointer; iSize: integer): boolean;
    function  ReceiveData( var iSize: integer ): pointer;
    function  CanRead( iTimeOutInSec: integer = 1 ): boolean;
    function  GetPort: string;
    procedure SetServerAddress( sAddr: string );
    function  GetServerAddress: string;
    function  GetSocketLastError: string;
    function  GetSocket: TSocket;
    procedure SetSocket( skt: TSocket );
    procedure SetPort( sPort: string );
    procedure CloseSocket;
  end;

// A THxBlockSocket encapsulates all the basic 'socket' functions.
  THxBlockSocket = class( TInterfacedObject, THxSocketInterface )
  protected
    sktSocket: TSocket;
    bConnected: boolean;
    rServerAddress: TSockAddrIn;
    iLastError: integer;
    sPortNr: string;
    sServerAddress: string;
    pRcvBuf: pointer;
    thrdOwnerThread: TBaseThread;
    FOnNotify: TOnNotify;
    FOnDisconnect: TOnDisconnect;
    procedure Notify( s: string );
    function  CanWrite: boolean; virtual;
  public
    constructor Create( thrdOwner: TBaseThread ); virtual;
    destructor Destroy; override;
    procedure Disconnect; virtual;
    function  Connect: integer; virtual;
    function  SendData( ptr: pointer; iSize: integer): boolean; virtual;
// NEVER free, dispose or whatever.. the result pointer from ReceiveData().!!
// It is maintained by this class.
    function  ReceiveData( var iSize: integer ): pointer; virtual;
    function  CanRead( iTimeOutInSec: integer = 1 ): boolean; virtual;
    function  GetPort: string; virtual;
    function  GetServerAddress: string; virtual;
    function  GetSocketLastError: string;
    function  GetSocket: TSocket;
    procedure SetServerAddress( sAddr: string );
    procedure CloseSocket; virtual;
    procedure SetSocket( skt: TSocket );
    procedure SetPort( sPort: string );
    property  OnNotify: TOnNotify read FOnNotify write FOnNotify;
    property  OnDisconnect: TOnDisconnect read FOnDisconnect write
              FOnDisconnect;
  end;

// THxBlockServer is a THxBlockSocket with some extra 'server' functionality
// added.
  THxBlockServer = class( THxBlockSocket )
  protected
    rRemoteAddress: TSockAddrIn;
    procedure SetListening( bValue: boolean );
    procedure Listen;
  public
    function Accept: TSocket;
    property Listening: boolean read bConnected write SetListening;
  end;

// A 'list' to store all connected TCP-sockets..
  TConnRecList = class( TList )
  protected
    procedure Put( index: integer; pElement: THxBlockSocket );
    function  Get( index: integer ): THxBlockSocket;
    procedure Notify( ptr: Pointer; Action: TListNotification ); override;
  public
    property items[ index: integer]: THxBlockSocket read Get
             write Put; default;
    procedure Add( ptr: THxBlockSocket );
  end;

  THxTcpServer = class;

// This thread will listen for incomming calls from remote TCP-clients.
// A sever component will maintaine only one instantion of this thread.
  THxConnThread = class( TBaseThread )
  protected
    lstConnections: TConnRecList;
    thdOwner: THxTcpServer;
    procedure Notify( s: string ); override;
    procedure ExecProc; override;
    procedure HandleData( p: pointer; iSize, iIdx: integer );
    procedure HandleDisconnect( blkS: THxBlockSocket );
    function  AnyThingToRead( var iIdx: integer ): boolean;
    function  NumActiveConnections: integer;
    function  GetIndexByBlockSocket( blkS: THxBlockSocket ): integer;
    procedure CheckDescriptor( iDescr: integer );
  public
    constructor Create( thd: THxTcpServer );
    destructor Destroy; override;
    function  AddConnection( blkS: THxBlockSocket ): integer;
    function  SendData( ptr: pointer; iSize, iDescr: integer ): boolean;
    function  GetServerAddress( iDescr: integer ): string;
    function  GetPort( iDescr: integer ): string;
  end;

  TOnNewClient = procedure( iDescr: integer ) of object;
  TOnClientAborted = procedure( iDescr: integer ) of object;
  TOnSvrDataAvailable = procedure( ptr: pointer;
                                   iSize, iDescr: integer ) of object;

// The TCP-Server class..
  THxTcpServer = class( TBaseThread )
  protected
    iLastNewClient: integer;
    iLastAbortedClient: integer;
    iLastDescr: integer;
    iLastMessSize: integer;
    pLastMessPtr: pointer;
    bsSvr: THxBlockServer;
    thdConnections: THxConnThread;
    FOnNewClient: TOnNewClient;
    FOnClientAborted: TOnClientAborted;
    FOnDataAvailable: TOnSvrDataAvailable;
    procedure AcceptConnection; virtual;
    function  GetPort: string;
    procedure SetPort(const Value: string);
    procedure NewClient;
    procedure NewClientSync( iDescr: integer );
    procedure ClientAborted;
    procedure ClientAbortedSync( iDescr: integer );
    procedure DataAvailable;
    procedure DataAvailableSync( ptr: pointer; iSize, iDescr: integer );
    procedure ExecProc; override;
    function  GetClientAddress( iDescr: Integer ): string; virtual;
    function  GetClientPort( iDescr: integer ): string; virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Listen;
    function SendData( ptr: pointer; iSize, iDescr: integer ): boolean; virtual;
    function SendString( s: string; iDescr: integer ): boolean;
    property ClientAddress[index: integer]: string read GetClientAddress;
    property ClientPort[index: integer]: string read GetClientPort;
    property Port: string read GetPort write SetPort ;
    property OnDataAvailable: TOnSvrDataAvailable read FOnDataAvailable
             write FOnDataAvailable;
    property OnClientAborted: TOnClientAborted read
             FOnClientAborted write FOnClientAborted;
    property OnNewClient: TOnNewClient read FOnNewClient write
             FOnNewClient;
  end;

  TOnDataAvailable = procedure( ptr: pointer; iSize: integer )of object;
  TOnConnectionClosed = procedure of object;

// The TCP-Client class..
  THxTcpClient = class( TBaseThread )
  protected
    ptrMess: pointer;
    iMessSize: integer;
    ifSock: THxSocketInterface;
    blkSock: THxBlockSocket;
    FOnDataAvailable: TOnDataAvailable;
    FOnConnectionClosed: TOnConnectionClosed;
    procedure SetConnected( const bValue: boolean );
    procedure SetAddress( sAddr: string );
    procedure SetPort( sPort: string );
    function  GetAddress: string;
    function  GetPort: string;
    function  GetConnected: boolean;
    procedure ExecProc; override;
    procedure DataAvailable;
    procedure ServerAborted;
    procedure HandleDisconnect( blkS: THxBlockSocket );
  public
    constructor Create;
    function  Connect: boolean;
    function  SendData( ptr: pointer; iSize: integer ): boolean;
    function  SendString( s: string ): boolean;
    property  Connected: boolean read GetConnected write SetConnected;
    property  ServerAddress: string read GetAddress write SetAddress;
    property  Port: string read GetPort write SetPort;
    property  OnDataAvailable: TOnDataAvailable read FOnDataAvailable
              write FOnDataAvailable;
    property  OnConnectionClosed: TOnConnectionClosed read FOnConnectionClosed
              write FOnConnectionClosed;
  end;

// Description of an ipc-header
  TIpcHeader = record
    aIpcMarker: array[0..Length( IPC_CLIENT )] of char; // + null char..
    aIpcVersion: array[0..Length( IPC_CL_VERSION)] of char; //  idem..
    iSizeOfMess: integer;
  end;
  PIpcHeader = ^TIpcHeader;

  TIpcVarHeader = record
    case boolean of
    True:
    ( rAsHeader: TIpcHeader;
    );
    False:
    (
      aAsArray: array[1..Sizeof( TIpcHeader)] of char;
    );
  end;

// Data structure to store IPC-messages..
  TIpcData = record
    rHeader: TIpcVarHeader;
    iFilledHeaderSize: integer;
    pMessage: pointer;
    pEndMessage: pointer;
    iMessSize: integer;
    iRemainder: integer;
    bPeerIsIPC: boolean;
  end;

// All IPC messages will be (temporary) stored in this datastructure
  TData = record
    bHandled: boolean;
    iSize: integer;
    ptr: pointer;
  end;
  PData = ^TData;

// A list of TData records..
  TDataList = class( TList )
  protected
    procedure Put( index: integer; pElement: PData );
    function  Get( index: integer ): PData;
    procedure Notify( ptr: Pointer; Action: TListNotification ); override;
  public
    property items[ index: integer]: PData read Get
             write Put; default;
    procedure Add( ptr: PData );
  end;

  THxIpcSocket = class;

  TOnIpcDisconnect = procedure( ipcSocket: THxIpcSocket )of object;

// THxIpcSocket is a THxBlockSocket with some extra functionality for
// separate message handling..
  THxIpcSocket = class( THxBlockSocket )
  protected
    lstData: TDataList;
    rIpcData: TIpcData;
    bServerSocket: boolean;
    FOnIpcDisconnect: TOnIpcDisconnect;
    procedure HandleNewMessage( ptr: pointer; iSize: integer);
    procedure FinishMessage( var ptr: pointer; var iSize: integer);
    function  ReturnQueuedMessage( var iSize: integer ): pointer;
    function  TestSvrMessage( var ptr: pointer; var iSize: integer ): boolean;
    function  TestClientMessage( var ptr: pointer;
                                 var iSize: integer ): boolean;
    function  CheckClientMessage( var ptr: pointer;
                                  var iSize: integer ): boolean;
  public
    constructor Create( thrdOwner: TBaseThread = nil ); override;
    destructor Destroy; override;
    procedure Disconnect; override;
    function  CanRead( iTimeOutInSec: integer = 1 ): boolean; override;
// NEVER free, dispose or whatever.. the result pointer from ReceiveData().!!
// It is maintained by this class.
    function  ReceiveData( var iSize: integer ): pointer; override;
    function  SendData( ptr: pointer; iSize: integer): boolean; override;
    function  Connect: integer; override;
    function  HasQueuedData: boolean;
    function  ConnectClient: boolean;
    property  OnIpcDisconnect: TOnIpcDisconnect read FOnIpcDisconnect
              write FOnIpcDisconnect;
  end;

// The ICP-Client class..
  THxIpcClient = class( THxTcpClient )
  protected
    ipcSock: THxIpcSocket;
    procedure HandleDisconnect( ipcS: THxIpcSocket );
  public
    constructor Create;
  end;

// A 'list' to store all connected ipc-sockets..
  TIpcConnRecList = class( TList )
  protected
    procedure Put( index: integer; pElement: THxIpcSocket );
    function  Get( index: integer ): THxIpcSocket;
    procedure Notify( ptr: Pointer; Action: TListNotification ); override;
  public
    property items[ index: integer]: THxIpcSocket read Get
             write Put; default;
    procedure Add( ptr: THxIpcSocket );
  end;

// This thread will listen for incomming calls from remote IPC-clients.
// A sever component will maintaine only one instantion of this thread.
  THxIpcConnThread = class( TBaseThread )
  protected
    lstConnections: TIpcConnRecList;
    thdOwner: THxTcpServer;
    procedure Notify( s: string ); override;
    procedure ExecProc; override;
    procedure HandleData( p: pointer; iSize, iIdx: integer );
    procedure HandleIpcDisconnect( ipcS: THxIpcSocket );
    function  AnyThingToRead( var iIdx: integer ): boolean;
    function  NumActiveConnections: integer;
    function  GetIndexByBlockSocket( ipcS: THxIpcSocket ): integer;
    procedure CheckDescriptor( iDescr: integer );
  public
    constructor Create( thd: THxTcpServer );
    destructor Destroy; override;
    function  AddConnection( ipcS: THxIpcSocket ): integer;
    function  SendData( ptr: pointer; iSize, iDescr: integer ): boolean;
    function  GetServerAddress( iDescr: integer ): string;
    function  GetPort( iDescr: integer ): string;
  end;

// The IPC-Server class..
  THxIpcServer = class( THxTcpServer )
  protected
    thdIpcConnections: THxIpcConnThread;
    procedure AcceptConnection; override;
    function  GetClientAddress( iDescr: Integer ): string; override;
    function  GetClientPort( iDescr: integer ): string; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    function SendData( ptr: pointer; iSize, iDescr: integer ): boolean; override;
  end;

// The functions below are redefined versions of the functions found in
// libc/winsock. I prefere the use of "pchar" above "var Buf"..
{$IFDEF LINUX}
function TcpRecv( fd: TSocket;
                  ptr: pchar;
                  iSize: size_t;
                  iFlags: integer ): integer; cdecl;

function TcpSend( fd: TSocket;
                  ptr: pchar;
                  iSize: size_t;
                  iFlags: integer ): integer; cdecl;

function UdpSendTo( fd: TSocket;
                    ptr: pchar;
                    iSize: size_t;
                    iFlags: integer;
                    var rDestinationSockAddr: TSockAddrIn;
                    iSocketAddrLen: integer ): integer; cdecl;

function UdpRecvFrom( fd: TSocket;
                      ptr: pchar;
                      iSize: size_t;
                      iFlags: integer;
                      var rSocketAddr: TSockAddr;
                      var iSocketAddrLen: integer ): integer; cdecl;
{$ELSE}
const
  winsocket = 'wsock32.dll';      { 32 bits TCP/IP system DLL }

type
  size_t = integer;

function TcpRecv( fd: TSocket;
                  ptr: pchar;
                  iSize: size_t;
                  iFlags: integer ): integer; stdcall;
function TcpSend( fd: TSocket;
                  ptr: pchar;
                  iSize: size_t;
                  iFlags: integer ): integer; stdcall;

function UdpSendTo( fd: TSocket;
                    ptr: pchar;
                    iSize: size_t;
                    iFlags: integer;
                    var rDestinationSockAddr: TSockAddr;
                    iSocketAddrLen: integer ): integer; stdcall;

function UdpRecvFrom( fd: TSocket;
                      ptr: pchar;
                      iSize: size_t;
                      iFlags: integer;
                      var rSocketAddr: TSockAddr;
                      var iSocketAddrLen: integer ): integer; stdcall;
{$ENDIF}

//---------------------------------------------------------------------------
// UDP stuff..
//---------------------------------------------------------------------------
const
  NO_LOCAL_ADDRESS_FOUND = '<NLF>';

type
// a THxUdpSocket encapsulates all the basic Udp-functionality..
  THxUdpSocket = class( THxBlockSocket )
  protected
    rRemoteAddress: TSockAddrIn;
    sRemoteAddress: string;
    sRemotePort: string;
    rRcvAddress: TSockAddr;
    sRcvAddress: string;
    sRcvPort: string;
    function BindSocket: boolean;
    procedure SetListening( bValue: boolean );
    procedure SetRemotePort( sPort: string );
    procedure SetRemoteAddress( sAddress: string );
  public
    bBound: boolean;
    constructor Create( thrdOwner: TBaseThread ); override;
    function  GetServerAddress: string; override;
    function  GetPort: string; override;
    procedure Disconnect; override;
    procedure SetReplyAddress;
    function  Connect: integer; override;
    function  SendData( ptr: pointer; iSize: integer): boolean; override;
// NEVER free, dispose or whatever.. the result pointer from ReceiveData().!!
// It is maintained by this class.
    function  ReceiveData( var iSize: integer ): pointer; override;
    property  RemotePort: string read sRemotePort write SetRemotePort;
    property  RemoteAddress: string read sRemoteAddress
              write SetRemoteAddress;
    property  ReceiveAddress: string read sRcvAddress;
    property  ReceivePort: string read sRcvPort;
    property  Listening: boolean read bBound write SetListening;
    property  OnNotify: TOnNotify read FOnNotify write FOnNotify;
  end;

  TOnUdpDataAvailable = procedure( pBuf: pchar; iSize: integer;
                                   sRemoteIP, sPort: string )of object;

// The UDP-Class..
  THxUdp = class( TBaseThread )
  protected
    ptrMess: pointer;
    iMessSize: integer;
    sListeningPort: string;
    ifSock: THxSocketInterface;
    udpSock: THxUdpSocket;
    FOnDataAvailable: TOnUdpDataAvailable;
    procedure SetDestinationAddress( sAddr: string );
    procedure SetDestinationPort( sPort: string );
    function  GetDestinationAddress: string;
    function  GetDestinationPort: string;
    function  GetRcvAddress: string;
    function  GetRcvPort: string;
    procedure SetListeningPort( sLP: string );
    procedure ExecProc; override;
    procedure DataAvailable;
    procedure Listen;
  public
    constructor Create;
    function  SendData( ptr: pointer; iSize: integer ): boolean;
    function  SendString( s: string ): boolean;
    function  Reply( ptr: pointer; iSize: integer ): boolean;
    function  ReplyStr( s: string ): boolean;
    property  DestinationAddress: string read GetDestinationAddress
              write SetDestinationAddress;
    property  DestinationPort: string read GetDestinationPort
              write SetDestinationPort;
    property  LastSendersAddress: string read GetRcvAddress;
    property  LastSendersPort: string read GetRcvPort;
    property  ListeningPort: string read sListeningPort write SetListeningPort;
    property  OnDataAvailable: TOnUdpDataAvailable read FOnDataAvailable
              write FOnDataAvailable;
  end;

implementation

{$IFDEF LINUX}
function TcpRecv;     external libcmodulename name 'recv';
function TcpSend;     external libcmodulename name 'send';
function UdpRecvFrom; external libcmodulename name 'recvfrom';
function UdpSendTo;   external libcmodulename name 'sendto';
{$ELSE}
function TcpRecv;     external winsocket name 'recv';
function TcpSend;     external winsocket name 'send';
function UdpRecvFrom; external winsocket name 'recvfrom';
function UdpSendTo;   external winsocket name 'sendto';
{$ENDIF}

{ TIpcConnRecList }

procedure TIpcConnRecList.Add( ptr: THxIpcSocket );
begin
  inherited Add( ptr );
end;

function TIpcConnRecList.Get( index: integer ): THxIpcSocket;
begin
  result := THxIpcSocket( inherited Get( index ));
end;

procedure TIpcConnRecList.Notify( ptr: Pointer; Action: TListNotification );
begin
  if Action = lnDeleted then
  begin
    if Assigned( ptr )then
      THxIpcSocket( ptr ).Free;
  end;
end;

procedure TIpcConnRecList.Put( index: integer; pElement: THxIpcSocket );
begin
  inherited Put( index, pElement );
end;

{ TConnRecList }

procedure TConnRecList.Add( ptr: THxBlockSocket );
begin
  inherited Add( ptr );
end;

function TConnRecList.Get( index: integer ): THxBlockSocket;
begin
  result := THxBlockSocket( inherited Get( index ));
end;

procedure TConnRecList.Notify( ptr: Pointer; Action: TListNotification );
begin
  if Action = lnDeleted then
  begin
    if Assigned( ptr )then
      THxBlockSocket( ptr ).Free;
  end;
end;

procedure TConnRecList.Put( index: integer; pElement: THxBlockSocket );
begin
  inherited Put( index, pElement );
end;

{ TBaseThread }

constructor TBaseThread.Create;
begin
  inherited Create( True ); // suspended..
  FreeOnTerminate := False;
  csLock := TCriticalSection.Create;
end;

destructor TBaseThread.Destroy;
begin
  inherited;
  csLock.Free;
end;

procedure TBaseThread.Execute;
begin
  try
    repeat
      ExecProc;
    until Terminated;
    sMess := 'ended';
  except
    on E: Exception do
      Notify('Exception: '+E.Message );
  end;
end;

procedure TBaseThread.Notify( s: string );
begin
  sMess := s;
  Synchronize( NotifySync );
end;

procedure TBaseThread.NotifySync;
begin
  if Assigned( FOnNotify )then
    FOnNotify( sMess );
end;


{ THxBlockSocket }

const
  UNSET = '<unset>';
  UNKNOWN = '<unknown>';

function IsNumber( sNumber: string ): Boolean;
var
  i: integer;
begin
  result := False;
  if sNumber = '' then  // Not a number..
    exit;

  for i := 1 to Length( sNumber )do
    if not( sNumber[i] in ['0'..'9'] )then
      exit;
  result := True;
end;


constructor THxBlockSocket.Create( thrdOwner: TBaseThread );
begin
  sktSocket := INVALID_SOCKET;
  sPortNr := UNSET;
  sServerAddress := UNKNOWN;
  pRcvBuf := nil;
  bConnected := False;
  thrdOwnerThread := thrdOwner;

  rServerAddress.sin_family := 0;
  rServerAddress.sin_addr.S_addr := 0;
end;

destructor THxBlockSocket.Destroy;
begin
  CloseSocket;

  if pRcvBuf <> nil then
    FreeMem( pRcvBuf );

  inherited;
end;

procedure THxBlockSocket.CloseSocket;
begin
  if sktSocket <> INVALID_SOCKET then
  begin
{$IFDEF LINUX}
    libc.shutdown( sktSocket, SHUT_RDWR );
{$ELSE}
    ShutDown( sktSocket, 2 );
{$ENDIF}
    bConnected := False;
    sktSocket := INVALID_SOCKET;
    sPortNr := UNSET;
    sServerAddress := UNKNOWN;
  end;
end;

procedure THxBlockSocket.Notify( s: string );
begin
  if Assigned( thrdOwnerThread )then
    thrdOwnerThread.Notify( s )
  else if Assigned( FOnNotify )then
    FOnNotify( s )
  else
    WriteLn( s );
end;

{$IFDEF LINUX}
function THxBlockSocket.GetSocketLastError: string;
begin
  result := strerror( errno );
end;
{$ELSE}
function THxBlockSocket.GetSocketLastError: string;
var
  iLastError: integer;
begin
  iLastError := WSAGetLastError;

  case iLastError of
    0 : result := 'No Error';
    WSAEINTR : result := 'Interrupted system call';
    WSAEBADF : result := 'Bad file number';
    WSAEACCES : result := 'Permission denied';
    WSAEFAULT : result := 'Bad address';
    WSAEINVAL : result := 'Invalid argument';
    WSAEMFILE : result := 'Too many open files';
    WSAEWOULDBLOCK : result := 'Operation would block';
    WSAEINPROGRESS : result := 'Operation now in progress';
    WSAEALREADY : result := 'Operation already in progress';
    WSAENOTSOCK : result := 'Socket operation on non-socket';
    WSAEDESTADDRREQ : result := 'Destination address required';
    WSAEMSGSIZE : result := 'Message too long';
    WSAEPROTOTYPE : result := 'Protocol wrong type for socket';
    WSAENOPROTOOPT : result := 'Protocol not available';
    WSAEPROTONOSUPPORT : result := 'Protocol not supported';
    WSAESOCKTNOSUPPORT : result := 'Socket type not supported';
    WSAEOPNOTSUPP : result := 'Operation not supported on socket';
    WSAEPFNOSUPPORT : result := 'Protocol family not supported';
    WSAEAFNOSUPPORT : result := 'Address family not supported '+
                                         'by protocol family';
    WSAEADDRINUSE : result := 'Address already in use';
    WSAEADDRNOTAVAIL  :  result := 'Address not available';
    WSAENETDOWN : result := 'Network is down';
    WSAENETUNREACH : result := 'Network is unreachable';
    WSAENETRESET : result := 'Network dropped connection on reset';
    WSAECONNABORTED : result := 'Connection aborted';
    WSAECONNRESET : result := 'Connection reset by peer';
    WSAENOBUFS : result := 'No buffer space available';
    WSAEISCONN : result := 'Socket is already connected';
    WSAENOTCONN : result := 'Socket is not connected';
    WSAESHUTDOWN : result := 'Can''t send after socket shutdown';
    WSAETOOMANYREFS : result := 'Too many references: can''t splice';
    WSAETIMEDOUT : result := 'Connection timed out';
    WSAECONNREFUSED : result := 'Connection refused';
    WSAELOOP : result := 'Too many levels of symbolic links';
    WSAENAMETOOLONG : result := 'File name too long';
    WSAEHOSTDOWN : result := 'Host is down';
    WSAEHOSTUNREACH : result := 'No route to host';
    WSAENOTEMPTY : result := 'Directory not empty';
    WSAEPROCLIM : result := 'Too many processes';
    WSAEUSERS : result := 'Too many users';
    WSAEDQUOT : result := 'Disc quota exceeded';
    WSAESTALE : result := 'Stale NFS file handle';
    WSAEREMOTE : result := 'Too many levels of remote in path';
    WSASYSNOTREADY : result := 'Network sub-system is unusable';
    WSAVERNOTSUPPORTED : result := 'WinSock DLL cannot support this '+
                                            'application';
    WSANOTINITIALISED : result := 'WinSock not initialized';
    WSAHOST_NOT_FOUND : result := 'Host not found';
    WSATRY_AGAIN : result := 'Non-authoritative host not found';
    WSANO_RECOVERY : result := 'Non-recoverable error';
    WSANO_DATA : result := '';
  else
    result := 'Not a WinSock error';
  end;
end;
{$ENDIF}

procedure THxBlockSocket.Disconnect;
begin
  if bConnected then
  begin
    CloseSocket;
    if Assigned( FOnDisconnect )then
      FOnDisconnect( Self );
  end;
end;

function THxBlockSocket.Connect: integer;
begin
{$IFDEF LINUX}
  result := libc.connect( sktSocket, rServerAddress, Sizeof( TSockAddrIn ));
{$ELSE}
  result := winsock.connect( sktSocket, rServerAddress, Sizeof( TSockAddrIn ));
{$ENDIF}
end;

function THxBlockSocket.GetServerAddress: string;
var
{$IFDEF LINUX}
  iSize: cardinal;
{$ELSE}
  iSize: integer;
{$ENDIF}
begin
  if sktSocket <> INVALID_SOCKET then
  begin
    iSize := Sizeof( TSockAddrIn );
    if getpeername( sktSocket, rServerAddress, iSize ) = -1 then
      raise Exception.Create( GetSocketLastError );

    result := Inet_NtoA( rServerAddress.sin_addr );
  end
  else
    result := sServerAddress;
end;

procedure THxBlockSocket.SetServerAddress( sAddr: string );
var
  pHe: PhostEnt;
  iInAdres: Dword;
begin
  sAddr := Trim( sAddr );
  if sAddr = '' then
    exit;

  iInAdres := Inet_addr( pchar( sAddr ));  // submitted as a dotted value.??

  if iInAdres <> INADDR_NONE then
  begin
    sServerAddress := sAddr;
    rServerAddress.sin_addr.s_addr := iInAdres;
  end
  else // Try to solve the hostname..
  begin
    pHe := GetHostByName( pchar( sAddr ));
    if pHe = nil then
       raise Exception.Create('Invalid address: '+sAddr );
    rServerAddress.sin_addr := PInAddr( pHe^.h_addr_list^ )^;
    sServerAddress := Inet_NtoA( rServerAddress.sin_addr );
  end;

  rServerAddress.sin_family := AF_INET;
end;

function THxBlockSocket.GetSocket: TSocket;
begin
  result := sktSocket;
end;

procedure THxBlockSocket.SetPort( sPort: string );
var
  pSe: PServEnt;
begin
  if bConnected then
    Disconnect;

  sPort := Trim( sPort );
  if sPort = '' then
    exit;

  if IsNumber( sPort ) then
  begin
    rServerAddress.sin_port := HtoNS( StrToInt( sPort ));
    sPortNr := sPort;
  end
  else
  begin
    pSe := GetServByName( pchar( sPort ), nil );
    if pSe = nil then
       raise Exception.Create('Invalid port: '+sPort );
    rServerAddress.sin_port := pSe^.s_port;
    sPort := IntToStr( NtoHS( pSe^.s_port ));
  end;
end;

function THxBlockSocket.GetPort: string;
var
{$IFDEF LINUX}
  iSize: cardinal;
{$ELSE}
  iSize: integer;
{$ENDIF}
begin
  if sktSocket <> INVALID_SOCKET then
  begin
    iSize := Sizeof( TSockAddrIn );
    if getpeername( sktSocket, rServerAddress, iSize ) = -1 then
      raise Exception.Create( GetSocketLastError );
    result := IntToStr( NtoHs( rServerAddress.sin_port ));
  end
  else
    result := sPortNr;
end;

function THxBlockSocket.CanRead( iTimeOutInSec: integer = 1 ): boolean;
var
  readfds: TFDSet;
  Timeout: TTimeVal;
  iRes: integer;
begin
  if sktSocket = INVALID_SOCKET then
  begin
    result := False;
    exit;
  end;
  
  Timeout.tv_sec := iTimeOutInSec;
  TimeOut.tv_usec := 0;

  FD_ZERO( readfds );
  FD_SET( sktSocket, readfds );

  iRes := select( sktSocket+1, @readfds, nil, nil, @Timeout );
  result := iRes > 0;
end;

function THxBlockSocket.CanWrite: boolean;
var
  writefds: TFDSet;
  Timeout: TTimeVal;
  iRes: integer;
begin
  FD_ZERO( writefds );
  FD_SET( sktSocket, writefds );

  Timeout.tv_sec := 0;
  TimeOut.tv_usec := 10;

  iRes := select( sktSocket+1, nil, @writefds, nil, @Timeout );

  result := iRes > 0;
end;

// NEVER free, dispose or whatever.. the result pointer.!!
// It is maintained by this class.
function THxBlockSocket.ReceiveData( var iSize: integer ): pointer;
var
  iRcvRes: integer;
begin
  if not( bConnected )then
    raise Exception.Create('No connection established');

  result := nil;

  if CanRead then
  begin
    iSize := 0;
{$IFDEF LINUX}
    if libc.ioctl( sktSocket, FIONREAD, @iSize )<> 0 then
{$ELSE}
    if ioctlSocket( sktSocket, FIONREAD, iSize )<> 0 then
{$ENDIF}
    begin
      Notify('Ioctl went wrong, message: '+GetSocketLastError );
      exit;
    end;

    if Assigned( pRcvBuf )then
      FreeMem( pRcvBuf );

    pRcvBuf := AllocMem( iSize );

    iRcvRes := TcpRecv( sktSocket, pRcvBuf, iSize, 0 );

    iSize := iRcvRes;

    if iRcvRes <= 0 then
      Disconnect
    else
      result := pRcvBuf;
  end;
end;

function THxBlockSocket.SendData( ptr: pointer;
                                  iSize: integer): boolean;
var
  iRes: integer;
begin
  if not( bConnected )then
    raise Exception.Create('No connection established');

  result := CanWrite;

  if result then
  begin
    iRes := TcpSend( sktSocket, ptr, iSize, 0 );
    if ( iRes <= 0 )then
    begin
      result := False;
      Disconnect;
    end
  end;
end;


{ THxTcpClient }

constructor THxTcpClient.Create;
begin
  inherited;

  blkSock := THxBlockSocket.Create( Self );
  blkSock.OnDisconnect := HandleDisconnect;
  ifSock := blkSock;
end;

function THxTcpClient.Connect: boolean;
var
  iRes: integer;
  sktSocket: TSocket;
begin
  result := True;
  if Connected then
    exit;

  if ServerAddress = UNSET then
    raise Exception.Create('Server-address not set.');

  if Port = UNSET then
    raise Exception.Create('Server-port not set');

  sktSocket := Socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );

  if sktSocket = INVALID_SOCKET then
    raise Exception.Create('Invalid socket returned, message: '+
                            ifSock.GetSocketLastError );


  ifSock.SetSocket( sktSocket );

  iRes := ifSock.Connect;

  result := iRes = 0;
  if result then
  begin
    if Suspended then
      Resume;
  end
  else
    ifSock.CloseSocket;
end;

procedure THxTcpClient.ServerAborted;
begin
  if Assigned( FOnConnectionClosed )then
    FOnConnectionClosed;
end;

procedure THxTcpClient.HandleDisconnect( blkS: THxBlockSocket );
begin
  Synchronize( ServerAborted );

// This is a very important check!
// Under (rare) situations it is possible that this procedure is called while
// this class is beeing destroyed. In this case it is better NOT to suspend the
// thread.
  if not Terminated then
    Suspend;
end;

procedure THxTcpClient.SetConnected( const bValue: boolean );
begin
  if bValue = Connected then
    exit; // Nothing to do..

  if bValue then
    Connect
  else
    ifSock.Disconnect; // this will 'fire' the eventhandler 'HandleDisconnect'
end;

function THxTcpClient.SendData( ptr: pointer; iSize: integer ): boolean;
begin
  result := ifSock.SendData( ptr, iSize );
end;

function THxTcpClient.SendString( s: string ): boolean;
begin
  result := SendData( pchar( s ), Length( s )+1 );
end;

function THxTcpClient.GetConnected: boolean;
begin
  result := ifSock.GetSocket <> INVALID_SOCKET;
end;

function THxTcpClient.GetAddress: string;
begin
  result := ifSock.GetServerAddress;
end;

procedure THxTcpClient.SetAddress( sAddr: string );
begin
  ifSock.SetServerAddress( sAddr );
end;

procedure THxTcpClient.SetPort( sPort: string );
begin
  ifSock.SetPort( sPort );
end;

function THxTcpClient.GetPort: string;
begin
  result := ifSock.GetPort;
end;

procedure THxTcpClient.ExecProc;
begin
  if ifSock.CanRead then
  begin
    ptrMess := ifSock.ReceiveData( iMessSize );
    if ptrMess <> nil then
      Synchronize( DataAvailable );
  end;
end;

procedure THxTcpClient.DataAvailable;
begin
  if Assigned( FOnDataAvailable )then
    FOnDataAvailable( ptrMess, iMessSize );
end;


{ THxBlockServer }

function THxBlockServer.Accept: TSocket;
var
  iLen: integer;
begin
  if sktSocket = INVALID_SOCKET then
    sktSocket := Socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );

  iLen := SizeOf( rRemoteAddress );
{$IFDEF LINUX}
  result := libc.accept( sktSocket, @rRemoteAddress, @iLen );
{$ELSE}
  result := winsock.accept( sktSocket, @rRemoteAddress, @iLen );
{$ENDIF}
end;

procedure THxBlockServer.Listen;
begin
  if sPortNr = UNSET then
    raise Exception.Create('Server-port not set');

  if sktSocket = INVALID_SOCKET then
    sktSocket := Socket( AF_INET, SOCK_STREAM, 0 );

  if sktSocket = INVALID_SOCKET then
    raise Exception.Create( GetSocketLastError );

  rServerAddress.sin_family := AF_INET;
  rServerAddress.sin_addr.S_addr := INADDR_ANY;

  if bind( sktSocket, rServerAddress, Sizeof( rServerAddress ))<> 0 then
    raise Exception.Create( GetSocketLastError );

{$IFDEF LINUX}
  if libc.listen( sktSocket, SOMAXCONN )<> 0 then
{$ELSE}
  if winsock.listen( sktSocket, SOMAXCONN )<> 0 then
{$ENDIF}
    raise Exception.Create( GetSocketLastError );

  bConnected := True;
end;

procedure THxBlockSocket.SetSocket( skt: TSocket );
begin
  if sktSocket <> INVALID_SOCKET then
    Disconnect;

  if skt <> INVALID_SOCKET then
  begin
    sktSocket := skt;
    bConnected := True;
  end;
end;

procedure THxBlockServer.SetListening( bValue: boolean );
begin
  if bValue = bConnected then
    exit;

  if bValue then
    Listen
  else
    Disconnect;
end;

{ THxTcpServer }

constructor THxTcpServer.Create;
begin
  inherited;

  iLastNewClient := -1;
  bsSvr := THxBlockServer.Create( Self );

  thdConnections := THxConnThread.Create( Self );
end;

destructor THxTcpServer.Destroy;
begin
  inherited;

  if Assigned( thdConnections )then
    thdConnections.Free;

  if Assigned( bsSvr )then
    bsSvr.Free;
end;

procedure THxTcpServer.ExecProc;
begin
  if( bsSvr.Listening )and( bsSvr.CanRead )then
    AcceptConnection;
end;

procedure THxTcpServer.AcceptConnection;
var
  blkS: THxBlockSocket;
  skt: TSocket;
  iDescr: integer;
begin
  skt := bsSvr.Accept;

  if skt = INVALID_SOCKET then
    exit;

  blkS := THxBlockSocket.Create( self );
  blkS.SetSocket( skt );

  iDescr := thdConnections.AddConnection( blkS );

  if thdConnections.Suspended then
    thdConnections.Resume;

  NewClientSync( iDescr );
end;

function THxTcpServer.GetPort: string;
begin
  result := bsSvr.GetPort;
end;

procedure THxTcpServer.SetPort(const Value: string);
begin
  if bsSvr.Listening then
  begin
    if not( Suspended )then
      Suspend;
    bsSvr.Listening := False;
    bsSvr.SetPort( Value );
    Listen;
  end
  else
    bsSvr.SetPort( Value );
end;

procedure THxTcpServer.Listen;
begin
  if Suspended then
  begin
    bsSvr.Listening := True;
    Resume;
  end;
end;

procedure THxTcpServer.NewClientSync( iDescr: integer );
begin
  iLastNewClient := iDescr;
  Synchronize( NewClient );
  iLastNewClient := -1;
end;

procedure THxTcpServer.ClientAbortedSync( iDescr: integer );
begin
  iLastAbortedClient := iDescr;
  Synchronize( ClientAborted );
  iLastAbortedClient := -1;
end;

procedure THxTcpServer.ClientAborted;
begin
  if Assigned( FOnClientAborted )then
    FOnClientAborted( iLastAbortedClient );
end;

procedure THxTcpServer.NewClient;
begin
  if Assigned( FOnNewClient )then
    FOnNewClient( iLastNewClient );
end;

procedure THxTcpServer.DataAvailable;
begin
  if Assigned( FOnDataAvailable )then
    FOnDataAvailable( pLastMessPtr, iLastMessSize, iLastDescr );
end;

procedure THxTcpServer.DataAvailableSync( ptr: pointer; iSize, iDescr: integer );
begin
  pLastMessPtr := ptr;
  iLastMessSize := iSize;
  iLastDescr := iDescr;

  Synchronize( DataAvailable );
end;

function THxTcpServer.SendString( s: string; iDescr: integer ): boolean;
begin
  result := SendData( pchar( s ), Length( s )+1, iDescr );
end;

function THxTcpServer.SendData( ptr: pointer; iSize, iDescr: integer ): boolean;
begin
  result := thdConnections.SendData( ptr, iSize, iDescr );
end;

function THxTcpServer.GetClientAddress( iDescr: Integer ): string;
begin
  result := thdConnections.GetServerAddress( iDescr );
end;

function THxTcpServer.GetClientPort( iDescr: integer ): string;
begin
  result := thdConnections.GetPort( iDescr );
end;


{ THxConnThread }

constructor THxConnThread.Create( thd: THxTcpServer );
begin
  lstConnections := TConnRecList.Create;

  thdOwner := thd;

  inherited Create;
end;

destructor THxConnThread.Destroy;
begin
  inherited;

  if Assigned( lstConnections )then
    lstConnections.Free;
end;

function THxConnThread.GetServerAddress( iDescr: integer ): string;
begin
  csLock.Acquire;
  try
    CheckDescriptor( iDescr );
    result := lstConnections[iDescr].GetServerAddress;
  finally
    csLock.Release;
  end;
end;

function THxConnThread.GetPort( iDescr: integer ): string;
begin
  csLock.Acquire;
  try
    CheckDescriptor( iDescr );
    result := lstConnections[iDescr].GetPort;
  finally
    csLock.Release;
  end;
end;

procedure THxConnThread.CheckDescriptor( iDescr: integer );
begin
  if( iDescr >= lstConnections.Count )or( iDescr < 0 )then
    raise Exception.Create('Invalid descriptor (out of bounds)');

  if lstConnections[iDescr].GetSocket = INVALID_SOCKET then
    raise Exception.CreateFmt('Connection is closed for descr: %d', [iDescr]);
end;

function THxConnThread.SendData( ptr: pointer; iSize, iDescr: integer ): boolean;
begin
  csLock.Acquire;
  try
    CheckDescriptor( iDescr );
    result := lstConnections[iDescr].SendData( ptr, iSize );
  finally
    csLock.Release;
  end;
end;

procedure THxConnThread.Notify( s: string );
begin
  if Assigned( thdOwner )then
    thdOwner.Notify( s );
end;

procedure THxConnThread.ExecProc;
var
  p: pointer;
  iSize, iIdx: integer;
begin
  iIdx := -1;
  if AnyThingToRead( iIdx )then
  begin
    if iIdx = -1 then
    begin
      Notify('Unexpected index returned' );
      exit;
    end;

    p := lstConnections[iIdx].ReceiveData( iSize );

    if p <> nil then
      HandleData( p, iSize, iIdx );
  end;
end;

function THxConnThread.AnyThingToRead( var iIdx: integer ): boolean;
var
  i, iRes: integer;
  sktMax, skt: TSocket;
  readfds: TFDSet;
  Timeout: TTimeVal;
  bLockSet: boolean;
begin
  csLock.Acquire;
  bLockSet := True;

  try
    Timeout.tv_sec := 1;
    TimeOut.tv_usec := 0;

    FD_ZERO( readfds );
    sktMax := -1;

    for i := 0 to lstConnections.Count -1 do
    begin
      skt := lstConnections[i].GetSocket;
      if skt <> INVALID_SOCKET then
      begin
        if skt > sktMax then
          sktMax := skt;
        FD_SET( skt, readfds );
      end;
    end;
    csLock.Release;
    bLockSet := False;

    iRes := select( sktMax+1, @readfds, nil, nil, @Timeout );
    result := iRes > 0;

    csLock.Acquire;
    bLockSet := True;
    if result then
    begin
      iIdx := -1;
      for i := 0 to lstConnections.Count -1 do
      begin
        skt := lstConnections[i].GetSocket;
        if skt <> INVALID_SOCKET then
        begin
          if FD_ISSET( skt, readfds )then
          begin
            iIdx := i;
            break;
          end;
        end;
      end;
    end;
    csLock.Release;
  except
    if bLockSet then
      csLock.Release;
    raise;
  end;
end;

function THxConnThread.AddConnection( blkS: THxBlockSocket ): integer;
var
  i: integer;
  bReUse: boolean;
begin
  csLock.Acquire;
  try
    result := -1; // keep the compiler happy..

    bReUse := False;
    for i := 0 to lstConnections.Count -1 do
    begin
      if lstConnections[i].GetSocket = INVALID_SOCKET then
      begin
//        lstConnections[i].Free;  Cleanup.. This is done implicitly..
        lstConnections[i] := blkS;  // and re-use
        bReUse := True;
        result := i;
        break;
      end;
    end;
    if not( bReUse )then
    begin
      lstConnections.Add( blkS );
      result := lstConnections.Count-1;
    end;

    blkS.OnDisconnect := HandleDisconnect;
  finally
    csLock.Release;
  end;
end;

procedure THxConnThread.HandleDisconnect( blkS: THxBlockSocket );
var
  iIdx: integer;
begin
  iIdx := GetIndexByBlockSocket( blkS );

  if Assigned( thdOwner )then
    thdOwner.ClientAbortedSync( iIdx );

  if( NumActiveConnections = 0 )and( not( Terminated ))then
    Suspend;
end;

function THxConnThread.NumActiveConnections: integer;
var
  i: integer;
begin
  result := 0;
  csLock.Acquire;
  try
    for i := 0 to lstConnections.Count -1 do
      if lstConnections[i].GetSocket <> INVALID_SOCKET then
        Inc( result );
  finally
    csLock.Release;
  end;
end;

function THxConnThread.GetIndexByBlockSocket( blkS: THxBlockSocket ): integer;
var
  i: integer;
begin
  csLock.Acquire;
  try
    for i := 0 to lstConnections.Count -1 do
    begin
       if lstConnections[i] = blkS then
       begin
         result := i;
         exit;
       end;
    end;
    result := -1;
  finally
    csLock.Release;
  end;
end;

procedure THxConnThread.HandleData( p: pointer; iSize, iIdx: integer );
begin
  if Assigned( thdOwner )then
    thdOwner.DataAvailableSync( p, iSize, iIdx );
end;

{ TDataList }

procedure TDataList.Add(ptr: PData);
begin
  inherited Add( ptr );
end;

function TDataList.Get(index: integer): PData;
begin
  result := PData( inherited Get( index ));
end;

procedure TDataList.Notify(ptr: Pointer; Action: TListNotification);
begin
  if Action = lnDeleted then
  begin
    ReAllocMem( PData( ptr ).ptr, 0 );
    Dispose( PData( ptr ));
  end;
end;

procedure TDataList.Put(index: integer; pElement: PData);
begin
  inherited Put( index, pElement );
end;

{ THxIpcSocket }

constructor THxIpcSocket.Create( thrdOwner: TBaseThread = nil );
begin
  inherited Create( thrdOwner );

  lstData := TDataList.Create;
  bServerSocket := False;
end;

destructor THxIpcSocket.Destroy;
begin
  lstData.Free;

  inherited;
end;

procedure THxIpcSocket.Disconnect;
begin
  if bConnected then
  begin
    CloseSocket;
    if Assigned( FOnIpcDisconnect )then
      FOnIpcDisconnect( Self )
  end;
end;

function THxIpcSocket.CanRead( iTimeOutInSec: integer = 1 ): boolean;
begin
  result := ( HasQueuedData )or( inherited CanRead( iTimeOutInsec ));
end;

function THxIpcSocket.HasQueuedData: boolean;
var
  i: integer;
begin
  result := False; // Until proven otherwise..
  if lstData.Count = 0 then
    exit;

  for i := 0 to lstData.Count -1 do
  begin
    if not( lstData[i]^.bHandled )then
    begin
      result := True;
      exit;
    end;
  end;
end;

function THxIpcSocket.ConnectClient: boolean;
var
  ptr: pointer;
  iSize: integer;
begin
  result := False;

  if not( CanRead( IPC_INTRO_TIMEOUT ))then
    exit;

  ptr := inherited ReceiveData( iSize );
  if ptr = nil then
    exit;

  rIpcData.bPeerIsIPC := CheckClientMessage( ptr, iSize );

  result := rIpcData.bPeerIsIPC;
end;


function THxIpcSocket.Connect: integer;
var
  iRes, iSize: integer;
  ptr: pointer;
begin
  iRes := inherited Connect;
  rIpcData.bPeerIsIPC := False;

  if iRes = 0 then
  begin
    SendData( nil, 0 );
    if CanRead( IPC_INTRO_TIMEOUT )then
    begin
      ptr :=  inherited ReceiveData( iSize );
      if Assigned( ptr )then
        rIpcData.bPeerIsIPC := TestSvrMessage( ptr, iSize );
    end
    else
      iRes := -2;
  end;
  if not( rIpcData.bPeerIsIPC )then
    iRes := -3;
  result := iRes;
end;

function THxIpcSocket.SendData( ptr: pointer; iSize: integer ): boolean;
var
  pMess: PIpcHeader;
  iNewSize: integer;
  pNewPtr: pchar;
begin
  if( not( rIpcData.bPeerIsIPC ))and( ptr <> nil )and( bConnected )then
  begin
    // Notify('No IPC-connection setup, the connection is shutdown.!' );
    Disconnect;
    result := False;
    exit;
  end;

  iNewSize := iSize + Sizeof( TIpcHeader );

  pMess := AllocMem( iNewSize );
  if bServerSocket then
  begin
    pMess^.aIpcMarker := IPC_SERVER;
    pMess^.aIpcVersion := IPC_SVR_VERSION;
  end
  else
  begin
    pMess^.aIpcMarker := IPC_CLIENT;
    pMess^.aIpcVersion := IPC_CL_VERSION;
  end;
  pMess^.iSizeOfMess := iSize;

  pNewPtr := pchar( integer( pMess )+Sizeof( TIpcHeader ));
{$IFDEF LINUX}
  if ptr <> nil then
    memcpy( pNewPtr, ptr, iSize );
{$ELSE}
  if ptr <> nil then
    CopyMemory( pNewPtr, ptr, iSize );
{$ENDIF}
  try
    result := inherited SendData( pMess, iNewSize );
  finally
    FreeMem( pMess );
  end;
end;

function THxIpcSocket.ReturnQueuedMessage( var iSize: integer ): pointer;
var
  i: integer;
begin
// Remove handled messages..
  i := lstData.Count -1;
  while i >= 0 do
  begin
    if lstData[i]^.bHandled then
      lstData.Delete( i );
    Dec( i );
  end;

// Are there any more messages in the queue..
  if lstData.Count > 0 then
  begin
    result := lstData[0]^.ptr;
    iSize := lstData[0]^.iSize;
    lstData[0]^.bHandled := True;
  end
  else
    result := nil;
end;

function THxIpcSocket.CheckClientMessage( var ptr: pointer;
                                          var iSize: integer ): boolean;
begin
  rIpcData.bPeerIsIPC := TestClientMessage( ptr, iSize );
  if not( rIpcData.bPeerIsIPC )then
  begin
    CloseSocket;
    ptr := nil;
    iSize := 0;
  end
  else
  begin
//    Notify('client accepted as ipc');
    bServerSocket := True;
    SendData( nil, 0 );
  end;
  result := rIpcData.bPeerIsIPC;
end;

// NEVER free, dispose or whatever.. the result pointer from ReceiveData().!!
// It is maintained by this class.
function THxIpcSocket.ReceiveData( var iSize: integer ): pointer;
var
  ptr: pointer;
begin
  result := ReturnQueuedMessage( iSize );
  if Assigned( result )then
    exit;

  ptr := inherited ReceiveData( iSize );
  if ptr = nil then
    exit;  // result already nil..

  if rIpcData.pMessage = nil then   // No,
    HandleNewMessage( ptr, iSize )  // Treat as new message..
  else
  begin
    FinishMessage( ptr, iSize ); // Process remainder..
    if iSize > 0 then
      HandleNewMessage( ptr, iSize ); // Process eventual other messages..
  end;

  result := ReturnQueuedMessage( iSize );
end;

procedure THxIpcSocket.HandleNewMessage( ptr: pointer; iSize: integer);
var
  pHeader: PIpcHeader;
  iDataSize, iSizeHeader: integer;
  p: PData;
begin
  if ptr = nil then
    exit;

  repeat
    if rIpcData.iFilledHeaderSize <> 0 then
    begin
      iSizeHeader := Sizeof( TIpcHeader )-rIpcData.iFilledHeaderSize;
// Copy sufficient bytes to the temporary header to fill it completely..
{$IFDEF LINUX}
      memcpy( @rIpcData.rHeader.aAsArray[rIpcData.iFilledHeaderSize+1],
              ptr, iSizeHeader );
{$ELSE}
      CopyMemory( @rIpcData.rHeader.aAsArray[rIpcData.iFilledHeaderSize+1],
              ptr, iSizeHeader );
{$ENDIF}
      if(
        ( bServerSocket )and
        ( rIpcData.rHeader.rAsHeader.aIpcMarker <> IPC_CLIENT ))or
        (( not( bServerSocket ))and
        ( rIpcData.rHeader.rAsHeader.aIpcMarker <> IPC_SERVER )
      )then
      begin
        // Notify('IPC-server sent an illegal (sub)message');
        exit;
      end;
// get the size of the message itself..
      rIpcData.iMessSize := rIpcData.rHeader.rAsHeader.iSizeOfMess;
      rIpcData.iFilledHeaderSize := 0; // mark as "not-in-use"
    end
    else
    begin
// See whether the message starts with an IPC-header
      pHeader := ptr;
      if(( bServerSocket )and( pHeader^.aIpcMarker <> IPC_CLIENT ))or
      (( not( bServerSocket ))and( pHeader^.aIpcMarker <> IPC_SERVER ))then
      begin
         Notify( Format('IPC-server sent an illegal message, iSize %d, '+
                'iMessSize: %d',[iSize, rIpcData.iMessSize]));
        exit;
      end;
// Get the size of the message itself.
      rIpcData.iMessSize := pHeader^.iSizeOfMess;
      iSizeHeader := Sizeof( TIpcHeader );
    end;

    rIpcData.iRemainder := rIpcData.iMessSize;

// Move ptr to the data area..
    Inc( Integer( ptr ), iSizeHeader );

// Get the remaining size of the message..
    iDataSize := iSize - iSizeHeader;

// Is the message complete??
// If yes store it in the list of messages..
    if rIpcData.iMessSize <= iDataSize then
    begin
      New( p );
      p^.ptr := nil;
      p^.bHandled := False;
      p^.iSize := rIpcData.iMessSize;
      ReAllocMem( p^.ptr, p^.iSize );
{$IFDEF LINUX}
      memcpy( p^.ptr, ptr, rIpcData.iMessSize );
{$ELSE}
      CopyMemory( p^.ptr, ptr, rIpcData.iMessSize );
{$ENDIF}
      lstData.Add( p );
// Are the any more messages in the queue??
// If yes than will remain iSize > 0..
      iSize := iSize - ( rIpcData.iMessSize + iSizeHeader );
// Move ptr to the start of the next message (if available)
      Inc( integer( ptr ), rIpcData.iMessSize );
// Arrived at this point, it can be (worst case) there is only a portion of
// the header available..
      if( iSize < Sizeof( TIpcHeader ))and( iSize <> 0 )then // Oeps..
      begin
{$IFDEF LINUX}
        memcpy( @rIpcData.rHeader.aAsArray[1], ptr, iSize );
{$ELSE}
        CopyMemory( @rIpcData.rHeader.aAsArray[1], ptr, iSize );
{$ENDIF}
        rIpcData.iFilledHeaderSize := iSize;
        iSize := 0;
      end;
    end
    else
    begin
// Hier aangekomen is het bericht (nog) niet volledig ingelezen..
// Reserveer ruimte voor het volledige bericht..
      rIpcData.pMessage := AllocMem( rIpcData.iMessSize );
// Kopieer het ingelezen gedeelte..
{$IFDEF LINUX}
      memcpy( rIpcData.pMessage, ptr, iDataSize );
{$ELSE}
      CopyMemory( rIpcData.pMessage, ptr, iDataSize );
{$ENDIF}
// Bereken de grootte van de rest van het bericht..
      rIpcData.iRemainder := rIpcData.iMessSize - iDataSize;
// onthoud waar de rest van het bericht geplaatst moet worden..
      rIpcData.pEndMessage := pointer( Integer( rIpcData.pMessage ) + iDataSize );

      break; // klaar..
    end;
  until iSize = 0;
end;

// Deze procedure wordt aangeroepen als aangenomen mag worden dat 'ptr' NIET
// op een bericht grens begint.
procedure THxIpcSocket.FinishMessage( var ptr: pointer; var iSize: integer);
var
  p: PData;
begin
  if ptr = nil then
    exit;

  if iSize < rIpcData.iRemainder then
  begin
{$IFDEF LINUX}
    memcpy( rIpcData.pEndMessage, ptr, iSize );
{$ELSE}
    CopyMemory( rIpcData.pEndMessage, ptr, iSize );
{$ENDIF}
    Inc( integer( rIpcData.pEndMessage ), iSize );
    Dec( rIpcData.iRemainder, iSize );
    iSize := 0;
  end
  else
// Het bericht is in zijn geheel aanwezig..
// Dus opslaan in de lijst met messages..
  begin
{$IFDEF LINUX}
    memcpy( rIpcData.pEndMessage, ptr, rIpcData.iRemainder );
{$ELSE}
    CopyMemory( rIpcData.pEndMessage, ptr, rIpcData.iRemainder );
{$ENDIF}
    Dec( iSize, rIpcData.iRemainder );
    Inc( integer( ptr ), rIpcData.iRemainder );

    New( p );
    p^.bHandled := False;
    p^.iSize := rIpcData.iMessSize;
    p^.ptr := nil;
    ReAllocMem( p^.ptr, p^.iSize );
{$IFDEF LINUX}
    memcpy( p^.Ptr, rIpcData.pMessage, p^.iSize );
{$ELSE}
    CopyMemory( p^.Ptr, rIpcData.pMessage, p^.iSize );
{$ENDIF}
    lstData.Add( p );

    FreeMem( rIpcData.pMessage );

    rIpcData.pMessage := nil;
    rIpcData.pEndMessage := nil;
    rIpcData.iMessSize := 0;
    rIpcData.iRemainder := 0;
    rIpcData.iFilledHeaderSize := 0;

// Hier aangekomen kan het zijn (worst-case) dat er alleen nog een gedeelte
// van een header aanwezig is..
    if( iSize < Sizeof( TIpcHeader ))and( iSize <> 0 )then // Oeps..
    begin
{$IFDEF LINUX}
      memcpy( @rIpcData.rHeader.aAsArray[1], ptr, iSize );
{$ELSE}
      CopyMemory( @rIpcData.rHeader.aAsArray[1], ptr, iSize );
{$ENDIF}
      rIpcData.iFilledHeaderSize := iSize;
      iSize := 0;
    end;
  end;
end;

function THxIpcSocket.TestSvrMessage( var ptr: pointer;
                                      var iSize: integer ): boolean;
var
  pHeader: PIpcHeader;
  sErrMess: string;
begin
  pHeader := ptr;
  sErrMess := '';

  if pHeader^.aIpcMarker = IPC_SERVER then
  begin
    if pHeader^.aIpcVersion <> IPC_SVR_VERSION then
      sErrMess := 'IPC-server has wrong version number'
    else if pHeader^.iSizeOfMess <> 0 then
      sErrMess := 'IPC-server sent an illegal message';
  end
  else
    sErrMess := 'Not connected to an IPC-server';

// Zet pointer op de eerste byte na het bericht..
  Inc( integer( ptr ), Sizeof( TIpcHeader ));
// Verminder de grootte met de grootte van het bericht.
  Dec( iSize, Sizeof( TIpcHeader ));

  if sErrMess <> '' then
    Notify( sErrMess );

  result := sErrMess = '';
  rIpcData.bPeerIsIPC := result;
end;

function THxIpcSocket.TestClientMessage( var ptr: pointer;
                                         var iSize: integer ): boolean;
var
  pHeader: PIpcHeader;
  sErrMess: string;
begin
  pHeader := ptr;
  sErrMess := '';

  if pHeader^.aIpcMarker = IPC_CLIENT then
  begin
    if pHeader^.aIpcVersion <> IPC_CL_VERSION then
      sErrMess := 'IPC-server has wrong version number'
    else if pHeader^.iSizeOfMess <> 0 then
      sErrMess := 'IPC-server sent an illegal message';
  end
  else
    sErrMess := 'Not connected to an IPC-client';

// Zet pointer op de eerste byte na het bericht..
  Inc( integer( ptr ), Sizeof( TIpcHeader ));
// Verminder de grootte met de grootte van het bericht.
  Dec( iSize, Sizeof( TIpcHeader ));

  if sErrMess <> '' then
    Notify( sErrMess );

  result := sErrMess = '';
  rIpcData.bPeerIsIPC := result;
end;

constructor THxIpcClient.Create;
begin
  inherited;

  ipcSock := THxIpcSocket.Create( Self );
  ipcSock.OnIpcDisconnect := HandleDisconnect;

  ifSock := ipcSock;
end;

procedure THxIpcClient.HandleDisconnect( ipcS: THxIpcSocket );
begin
  Synchronize( ServerAborted );
  if not( Terminated )then
    Suspend;
end;

{ THxIpcConnThread }

constructor THxIpcConnThread.Create( thd: THxTcpServer );
begin
  lstConnections := TIpcConnRecList.Create;

  thdOwner := thd;

  inherited Create;
end;

destructor THxIpcConnThread.Destroy;
begin
  inherited;

  if Assigned( lstConnections )then
    lstConnections.Free;
end;

function THxIpcConnThread.GetServerAddress( iDescr: integer ): string;
begin
  csLock.Acquire;
  try
    CheckDescriptor( iDescr );
    result := lstConnections[iDescr].GetServerAddress;
  finally
    csLock.Release;
  end;
end;

function THxIpcConnThread.GetPort( iDescr: integer ): string;
begin
  csLock.Acquire;
  try
    CheckDescriptor( iDescr );
    result := lstConnections[iDescr].GetPort;
  finally
    csLock.Release;
  end;
end;

procedure THxIpcConnThread.CheckDescriptor( iDescr: integer );
begin
  if( iDescr >= lstConnections.Count )or( iDescr < 0 )then
    raise Exception.Create('Invalid descriptor (out of bounds)');

  if lstConnections[iDescr].GetSocket = INVALID_SOCKET then
    raise Exception.CreateFmt('Connection is closed for descr: %d', [iDescr]);
end;

function THxIpcConnThread.SendData( ptr: pointer; iSize,
                                    iDescr: integer ): boolean;
begin
  csLock.Acquire;
  try
    CheckDescriptor( iDescr );

    result := lstConnections[iDescr].SendData( ptr, iSize );
  finally
    csLock.Release;
  end;
end;

procedure THxIpcConnThread.Notify( s: string );
begin
  if Assigned( thdOwner )then
    thdOwner.Notify( s );
end;

function THxIpcConnThread.AnyThingToRead( var iIdx: integer ): boolean;
var
  i, iRes: integer;
  sktMax, skt: TSocket;
  readfds: TFDSet;
  Timeout: TTimeVal;
begin
  Timeout.tv_sec := 1;
  TimeOut.tv_usec := 0;

  FD_ZERO( readfds );
  sktMax := -1;

  for i := 0 to lstConnections.Count -1 do
  begin
    if lstConnections[i].HasQueuedData then
    begin
      iIdx := i;
      result := True;
      exit;
    end;
    skt := lstConnections[i].GetSocket;
    if skt <> INVALID_SOCKET then
    begin
      if skt > sktMax then
        sktMax := skt;
      FD_SET( skt, readfds );
    end;
  end;

  iRes := select( sktMax+1, @readfds, nil, nil, @Timeout );
  result := iRes > 0;

  if result then
  begin
    iIdx := -1;
    for i := 0 to lstConnections.Count -1 do
    begin
      skt := lstConnections[i].GetSocket;
      if skt <> INVALID_SOCKET then
      begin
        if FD_ISSET( skt, readfds )then
        begin
          iIdx := i;
          break;
        end;
      end;
    end;
  end;
end;

procedure THxIpcConnThread.ExecProc;
var
  p: pointer;
  iSize, iIdx: integer;
begin
  iIdx := -1;

  if AnyThingToRead( iIdx )then
  begin
    if iIdx = -1 then
    begin
      Notify('Unexpected index returned' );
      exit;
    end;

    p := lstConnections[iIdx].ReceiveData( iSize );

    if p <> nil then
      HandleData( p, iSize, iIdx );
  end;
end;

// This function is called from within the context of the THxTcpServer thread.
function THxIpcConnThread.AddConnection( ipcS: THxIpcSocket ): integer;
var
  i: integer;
  bReUse: boolean;
begin
  csLock.Acquire;
  try
    result := -1; // keep the compiler happy..

    bReUse := False;
    for i := 0 to lstConnections.Count -1 do
    begin
      if lstConnections[i].GetSocket = INVALID_SOCKET then
      begin
//        lstConnections[i].Free;  This cleanup is done implicitly..
        lstConnections[i] := ipcS;  // and re-use..
        bReUse := True;
        result := i;
        break;
      end;
    end;
    if not( bReUse )then
    begin
      lstConnections.Add( ipcS );
      result := lstConnections.Count-1;
    end;

    ipcS.OnIpcDisconnect := HandleIpcDisconnect;

  finally
    csLock.Release;
  end;
end;

procedure THxIpcConnThread.HandleIpcDisconnect( ipcS: THxIpcSocket );
var
  iIdx: integer;
begin
  iIdx := GetIndexByBlockSocket( ipcS );

  if Assigned( thdOwner )then
    thdOwner.ClientAbortedSync( iIdx );

  if( NumActiveConnections = 0 )and( not( Terminated ))then
    Suspend;
end;

function THxIpcConnThread.NumActiveConnections: integer;
var
  i: integer;
begin
  csLock.Acquire;
  try
    result := 0;
    for i := 0 to lstConnections.Count -1 do
      if lstConnections[i].GetSocket <> INVALID_SOCKET then
        Inc( result );
  finally
    csLock.Release;
  end;
end;

function THxIpcConnThread.GetIndexByBlockSocket( ipcS: THxIpcSocket ): integer;
var
  i: integer;
begin
  csLock.Acquire;
  try
    for i := 0 to lstConnections.Count -1 do
    begin
       if lstConnections[i] = ipcS then
       begin
         result := i;
         exit;
       end;
    end;
    result := -1;
  finally
    csLock.Release;
  end;
end;

procedure THxIpcConnThread.HandleData( p: pointer; iSize, iIdx: integer );
begin
  if Assigned( thdOwner )then
    thdOwner.DataAvailableSync( p, iSize, iIdx );
end;

{  THxIpcServer }

constructor THxIpcServer.Create;
begin
  inherited;

  iLastNewClient := -1;
  bsSvr := THxBlockServer.Create( Self );

  thdIpcConnections := THxIpcConnThread.Create( Self );
end;

destructor THxIpcServer.Destroy;
begin
  inherited;

  if Assigned( thdIpcConnections )then
    thdIpcConnections.Free;
end;

procedure THxIpcServer.AcceptConnection;
var
  ipcS: THxIpcSocket;
  skt: TSocket;
  iDescr: integer;
  bValid: boolean;
begin
  skt := bsSvr.Accept;

  if skt = INVALID_SOCKET then
    exit;

  ipcS := THxIpcSocket.Create( self );
  ipcS.SetSocket( skt );

  bValid := ipcS.ConnectClient;

  if not( bValid )then
    ipcS.CloseSocket;

  iDescr := thdIpcConnections.AddConnection( IpcS );

  if bValid then
  begin
    if thdIpcConnections.Suspended then
      thdIpcConnections.Resume;

    NewClientSync( iDescr );
  end;
end;

function THxIpcServer.SendData( ptr: pointer; iSize,
                                iDescr: integer ): boolean;
begin
  result := thdIpcConnections.SendData( ptr, iSize, iDescr );
end;

function THxIpcServer.GetClientAddress( iDescr: Integer ): string;
begin
  result := thdIpcConnections.GetServerAddress( iDescr );
end;

function THxIpcServer.GetClientPort( iDescr: integer ): string;
begin
  result := thdIpcConnections.GetPort( iDescr );
end;

{ THxUdpSocket }

function THxUdpSocket.Connect: integer;
begin
  result := 0;
end;

procedure THxUdpSocket.Disconnect;
begin
// Do nothing..
end;

function THxUdpSocket.GetPort: string;
begin
  if( sktSocket <> INVALID_SOCKET )and
  ( rServerAddress.sin_addr.S_addr <> 0 ) then
    result := IntToStr( NtoHs( rServerAddress.sin_port ))
  else
    result := UNSET;
end;

function THxUdpSocket.GetServerAddress: string;
begin
  if( sktSocket <> INVALID_SOCKET )and
  ( rServerAddress.sin_addr.S_addr <> 0 )then
    result := Inet_NtoA( rServerAddress.sin_addr )
  else
    result := UNSET;
end;

// NEVER free, dispose or whatever.. the result pointer from ReceiveData().!!
// It is maintained by this class.
function THxUdpSocket.ReceiveData( var iSize: integer ): pointer;
var
  iRAlen, iRcvRes: integer;
begin
  result := nil;

  if CanRead then
  begin
    iSize := 0;
{$IFDEF LINUX}
    if libc.ioctl( sktSocket, FIONREAD, @iSize )<> 0 then
{$ELSE}
    if ioctlSocket( sktSocket, FIONREAD, iSize )<> 0 then
{$ENDIF}
    begin
      Notify('Ioctl went wrong, message: '+GetSocketLastError );
      exit;
    end;

    if Assigned( pRcvBuf )then
      FreeMem( pRcvBuf );

    pRcvBuf := AllocMem( iSize );

    iRALen := Sizeof( rRcvAddress );
    iRcvRes := UdpRecvFrom( sktSocket, pRcvBuf, iSize, 0,
                            rRcvAddress, iRALen );
    if iRcvRes < 0 then
      Notify( GetSocketLastError )
    else
    begin
      sRcvAddress := Inet_NtoA( rRcvAddress.sin_addr );
      sRcvPort := IntToStr( NtoHs( rRcvAddress.sin_port ));
    end;

    iSize := iRcvRes;

    if iRcvRes <= 0 then
      result := nil
    else
      result := pRcvBuf;
  end;

end;

function THxUdpSocket.SendData(ptr: pointer; iSize: integer): boolean;
var
  iRes: integer;
begin
  iRes := UdpSendTo( sktSocket, ptr, iSize, 0, rRemoteAddress,
                       Sizeof( rRemoteAddress ));
  result := iRes >= 0;
end;

function THxUdpSocket.BindSocket: boolean;
begin
  result := False;
  if sPortNr = UNSET then
    raise Exception.Create('UDP Server-port not set');

  if sktSocket = INVALID_SOCKET then
    sktSocket := Socket( AF_INET, SOCK_STREAM, 0 );

  if sktSocket = INVALID_SOCKET then
  begin
    Notify( GetSocketLastError );
    exit;
  end;

  rServerAddress.sin_family := AF_INET;
  rServerAddress.sin_addr.S_addr := INADDR_ANY;

  if bind( sktSocket, rServerAddress, Sizeof( rServerAddress ))<> 0 then
  begin
    Notify( GetSocketLastError );
    exit;
  end;
  result := True;
end;

procedure THxUdpSocket.SetListening( bValue: boolean );
begin
  if bValue = bBound then
    exit;

  if bValue then
    bBound := BindSocket
  else
  begin
    CloseSocket;
    bBound := False;
  end;
end;

procedure THxUdpSocket.SetRemoteAddress( sAddress: string );
var
  pHe: PhostEnt;
  iInAdres: Dword;
begin
  sAddress := Trim( sAddress );
  if sAddress = '' then
    exit;

  iInAdres := Inet_addr( pchar( sAddress ));  // submitted as a dotted value.??

  if iInAdres <> INADDR_NONE then
  begin
    sRemoteAddress := sAddress;
    rRemoteAddress.sin_addr.s_addr := iInAdres;
  end
  else // Try to solve the hostname..
  begin
    pHe := GetHostByName( pchar( sAddress ));
    if pHe = nil then
       raise Exception.Create('Invalid address: '+sAddress );
    rRemoteAddress.sin_addr := PInAddr( pHe^.h_addr_list^ )^;
    sRemoteAddress := Inet_NtoA( rRemoteAddress.sin_addr );
  end;

  rRemoteAddress.sin_family := AF_INET;
end;

procedure THxUdpSocket.SetRemotePort( sPort: string );
var
  pSe: PServEnt;
begin
  if bConnected then
    Disconnect;

  sPort := Trim( sPort );
  if sPort = '' then
    exit;

  if IsNumber( sPort ) then
  begin
    rRemoteAddress.sin_port := HtoNS( StrToInt( sPort ));
    sRemotePort := sPort;
  end
  else
  begin
    pSe := GetServByName( pchar( sPort ), nil );
    if pSe = nil then
       raise Exception.Create('Invalid port: '+sPort );
    rRemoteAddress.sin_port := pSe^.s_port;
    sRemotePort := IntToStr( NtoHS( pSe^.s_port ));
  end;
end;

constructor THxUdpSocket.Create( thrdOwner: TBaseThread );
begin
  inherited;

  sRemoteAddress := UNSET;
  sRemotePort := UNSET;
  sRcvAddress := UNSET;
end;

procedure THxUdpSocket.SetReplyAddress;
begin
  if sRcvAddress <> UNSET then
    rRemoteAddress := rRcvAddress;
end;

{ THxUdp }

constructor THxUdp.Create;
var
  sktSocket: TSocket;
begin
  inherited;

  udpSock := THxUdpSocket.Create( Self );
  ifSock := udpSock;

  sktSocket := Socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );

  if sktSocket = INVALID_SOCKET then
    raise Exception.Create('Invalid udp-socket returned, message: '+
                            ifSock.GetSocketLastError );
  ifSock.SetSocket( sktSocket );
end;

procedure THxUdp.DataAvailable;
begin
  if Assigned( FOnDataAvailable )then
    FOnDataAvailable( ptrMess, iMessSize, LastSendersAddress,
                      LastSendersPort );
end;

procedure THxUdp.ExecProc;
begin
  if ifSock.CanRead then
  begin
    ptrMess := ifSock.ReceiveData( iMessSize );
    if ptrMess <> nil then
      Synchronize( DataAvailable );
  end;
end;

function THxUdp.GetDestinationAddress: string;
begin
  result := udpSock.RemoteAddress;
end;

function THxUdp.GetDestinationPort: string;
begin
  result := udpSock.RemotePort;
end;

function THxUdp.GetRcvAddress: string;
begin
  result := udpSock.ReceiveAddress;
end;

function THxUdp.GetRcvPort: string;
begin
  result := udpSock.ReceivePort;
end;

procedure THxUdp.Listen;
begin
  if Suspended then
  begin
    udpSock.SetListening( True );
    if udpSock.Listening then
      Resume;
  end;
end;

function THxUdp.SendData( ptr: pointer; iSize: integer ): boolean;
begin
  if DestinationAddress = UNSET then
    raise Exception.Create('Destination-address not set.');

  if DestinationPort  = UNSET then
    raise Exception.Create('Destination-port not set');

  result := ifSock.SendData( ptr, iSize );
end;

function THxUdp.SendString( s: string ): boolean;
begin
  result := SendData( pchar( s ), Length( s )+1 );
end;

function THxUdp.Reply( ptr: pointer; iSize: integer ): boolean;
begin
  udpSock.SetReplyAddress;
  result := SendData( ptr, iSize );
end;

function THxUdp.ReplyStr( s: string ): boolean;
begin
  result := Reply( pchar( s ), Length( s )+1 );
end;

procedure THxUdp.SetListeningPort( sLP: string );
var
  sktSocket: TSocket;
begin
  if not( Suspended )then
    Suspend;

  udpSock.Listening := False;
  ifSock.SetPort( sLP );

  sktSocket := Socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );

  if sktSocket = INVALID_SOCKET then
    raise Exception.Create('Invalid udp-socket returned, message: '+
                            ifSock.GetSocketLastError );

  ifSock.SetSocket( sktSocket );

  sListeningPort := sLP;

  Listen;
end;

procedure THxUdp.SetDestinationAddress( sAddr: string );
begin
  udpSock.RemoteAddress := sAddr;
end;

procedure THxUdp.SetDestinationPort( sPort: string );
begin
  udpSock.RemotePort := sPort;
end;

{$IFNDEF LINUX}
procedure LoadDll;
var
  rInitData : TWSADATA;
  iLastError: integer;
begin
  iLastError := WSAStartup( $101, rInitData );  // Werk winsock met v1.1
  if iLastError <> 0 then
     raise Exception.CreateFmt('WSAStartup error #%d',
                                [iLastError]);
end;

initialization

LoadDLL;
{$ENDIF}

end.
